@@ -93,6 +93,12 @@ finalize (GObject *object)
 
 E2K_MAKE_TYPE (exchange_hierarchy_favorites, ExchangeHierarchyFavorites, class_init, init, PARENT_TYPE)
 
+static void
+add_hrefs (ExchangeHierarchy *hier, EFolder *folder, gpointer hrefs)
+{
+	g_ptr_array_add (hrefs, (char *)e2k_uri_path (e_folder_exchange_get_internal_uri (folder)));
+}
+
 static const char *shortcuts_props[] = {
 	PR_FAV_DISPLAY_NAME,		/* PR_DISPLAY_NAME of referent */
 	PR_FAV_DISPLAY_ALIAS,		/* if set, user-chosen display name */
@@ -119,6 +125,10 @@ get_hrefs (ExchangeHierarchySomeDAV *hsd)
 
 	hrefs = g_ptr_array_new ();
 
+	if (exchange_account_is_offline (hier->account)) {
+		exchange_hierarchy_webdav_offline_scan_subtree (EXCHANGE_HIERARCHY (hfav), add_hrefs, hrefs);
+		return hrefs;
+	}
 	/* Scan the shortcut links and use PROPFIND to resolve the
 	 * permanent_urls. Unfortunately it doesn't seem to be possible
 	 * to BPROPFIND a group of permanent_urls.
@@ -166,9 +176,6 @@ remove_folder (ExchangeHierarchy *hier, EFolder *folder)
 		EXCHANGE_HIERARCHY_FAVORITES (hier);
 	const char *folder_uri, *shortcut_uri;
 	E2kHTTPStatus status;
-	char *conf_key_cal="/apps/evolution/calendar/sources";
-	char *conf_key_tasks="/apps/evolution/tasks/sources";
-	char *conf_key_contacts="/apps/evolution/addressbook/sources";
 	ESourceList *cal_source_list, *task_source_list, *cont_source_list;
 	const char *folder_type, *physical_uri;
 
@@ -194,27 +201,27 @@ remove_folder (ExchangeHierarchy *hier, EFolder *folder)
 	if (strcmp (folder_type, "calendar") == 0) {
 		cal_source_list = e_source_list_new_for_gconf (
 					gconf_client_get_default (),
-					conf_key_cal);
-		remove_esource (hier->account, conf_key_cal, physical_uri,
-				&cal_source_list, FALSE, FALSE);
+					CONF_KEY_CAL);
+		remove_esource (hier->account, EXCHANGE_CALENDAR_FOLDER, 
+				physical_uri, &cal_source_list, FALSE);
 		e_source_list_sync (cal_source_list, NULL);
 		g_object_unref (cal_source_list);
 	}
 	else if (strcmp (folder_type, "tasks") == 0) {
 		task_source_list = e_source_list_new_for_gconf (
 					gconf_client_get_default (),
-					conf_key_tasks);
-		remove_esource (hier->account, conf_key_tasks, physical_uri,
-				&task_source_list, FALSE, FALSE);
+					CONF_KEY_TASKS);
+		remove_esource (hier->account, EXCHANGE_TASKS_FOLDER, 
+				physical_uri, &task_source_list, FALSE);
 		e_source_list_sync (task_source_list, NULL);
 		g_object_unref (task_source_list);
 	}
 	else if (strcmp (folder_type, "contacts") == 0) {
 		cont_source_list = e_source_list_new_for_gconf (
 					gconf_client_get_default (),
-					conf_key_contacts);
-		remove_esource (hier->account, conf_key_contacts, physical_uri,
-					&cont_source_list, FALSE, TRUE);
+					CONF_KEY_CONTACTS);
+		remove_esource (hier->account, EXCHANGE_CONTACTS_FOLDER, 
+				physical_uri, &cont_source_list, FALSE);
 		e_source_list_sync (cont_source_list, NULL);
 		g_object_unref (cont_source_list);
 	}
